/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the Network Research
 * 	Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: load_stat.h,v 1.3 96/02/15 17:30:37 kfall Exp $ (LBL)
 */

#include <sys/types.h>
#include "sys-time.h"

class LoadStats {
	timeval	start_decode_;	// time decoder started
	timeval start_render_;	// time decoder done / render started
	timeval done_render_;	// time render done
	int ndblks_;		// # blks decoded by decoder
public:
	inline void sd(const timeval& t) { start_decode_ = t; }
	inline void sr(const timeval& t) { start_render_ = t; }
	inline void nd(const int nblocks) { ndblks_ = nblocks; }
	inline void dr(const timeval& t) { done_render_ = t;
#ifdef DEBUGLOAD
		printf("%g %g %d\n",
			tvsub(start_render_, start_decode_),
			tvsub(done_render_, start_render_),
			ndblks_
		);
#endif
	}
	inline LoadStats() {
		memset(&start_decode_, 0, sizeof(timeval));
		memset(&start_render_, 0, sizeof(timeval));
		memset(&done_render_, 0, sizeof(timeval));
		ndblks_ = 0;
	}

private:
	inline double tvsub(timeval& t1, timeval& t2) {
		double t00 = t1.tv_sec + t1.tv_usec / 1000000.;
		double t01 = t2.tv_sec + t2.tv_usec / 1000000.;
		return(t00-t01);
	}
};
